/*
 * Block.h
 *
 *  Created on: Apr 1, 2011
 *      Author: lipari
 */

#ifndef BLOCK_H_
#define BLOCK_H_

#include <vector>
#include "Buffer.h"


class Visitor;

class Block {
	virtual void intComputeState() {}
	virtual void intComputeOutput() = 0;
	virtual bool intIsStateFul() = 0;
protected:

	Buffer outputs;
	std::vector<InputPort> inputs;

	Block(const Block &b);
	Block &operator=(const Block &);
public:
	Block(unsigned n_in, unsigned n_out);

	void computeState() { intComputeState(); }
	void computeOutput() { intComputeOutput(); }
	bool isStateFul() { return intIsStateFul(); }
	virtual Block *clone() { return 0; }
	virtual void accept(Visitor *v) = 0;

	InputPort &getInputPort(unsigned i) { return inputs[i]; }
	Buffer& getOutputBuffer() { return outputs; }

	std::vector<Block *> getPreviousBlocks();

	virtual ~Block() {}
};


class Stateless : public Block {
	virtual bool intIsStateFul() { return false; }
public:
	Stateless(unsigned n_in, unsigned n_out) : Block (n_in, n_out) {}
	void accept(Visitor *v);
};

class Stateful : public Block {
	virtual bool intIsStateFul() { return true; }
public:
	Stateful(unsigned n_in, unsigned n_out) : Block (n_in, n_out) {}
	void accept(Visitor *v);
};

#endif /* BLOCK_H_ */
